--[[
	### ItemValue ###
	ItemValue StatSetFormula Class
	
--]]

-- Libraries
local L = AceLibrary("AceLocale-2.2"):new("ItemValue")
local AceOO = AceLibrary("AceOO-2.0")
local ItemBonusLib = AceLibrary("ItemBonusLib-1.0")

local db

local Formula = AceOO.Class()

function Formula.LoadSets()
	db = ItemValue:AcquireDBNamespace("Formula")
	
	for SetName in pairs(db.profile) do
		ItemValue:RegisterSet(Formula:new(SetName))
	end
end

function Formula.prototype:init(Name, argFormula)
	Formula.super.prototype.init(self)
	
	if not db.profile[Name] then
		db.profile[Name] = {
			Formula = argFormula or "",
		}
	end
	
	self.Name = Name
	self.Formula = db.profile[Name].Formula
	
	self.Options = {
		type = "group",
		name = Name,
		desc = string.format(L["Settings for Set \"%s\"."], Name),
		args = {
			formula = {
				order = 100,
				type  = "text",
				name  = L["Formula"],
				desc  = L["Formula"],
				usage = L["<Formula>"],
				get   = function() return self.Formula or "" end,
				set   = function(v)
					self.Formula = v
					db.profile[Name].Formula = v
					ItemValue:ClearCache()
				end
			},
		},
	}

end

function Formula.prototype:tostring()
	return "ItemValue Formula " .. self.Name
end

function Formula.prototype:OnDelete()
	db.profile[self.Name] = nil
end

local meta = {["__index"] = function() return 0 end}
function Formula.prototype:GetEquipValue(eq)
	Formula.Data = setmetatable(eq, meta)
	local Points = loadstring("local math=math;setfenv(1, ItemValue.SetTypes.Formula.Data);return " .. self.Formula)()
	setmetatable(eq, nil)
	Formula.Data = nil
	
	return Points or 0
end

function Formula.prototype:GetItemValue(ItemString)
	return self:GetEquipValue(ItemBonusLib:ScanItem(ItemString, true))
end

function Formula.prototype:Serialize()
	return {Type = "Formula", Name = self.Name, Formula = self.Formula}
end

function Formula:Deserialize(t)
	return Formula:new(t.Name, t.Formula)
end

ItemValue:RegisterSetType("Formula", Formula)
