--[[
	### ItemValue ###
	ItemValue StatSet Class
	
--]]

-- Libraries
local L = AceLibrary("AceLocale-2.2"):new("ItemValue")
local AceOO = AceLibrary("AceOO-2.0")
local ItemBonusLib = AceLibrary("ItemBonusLib-1.0")

local db

local Group = AceOO.Class()

function Group.LoadSets()
	db = ItemValue:AcquireDBNamespace("Group")
	
	for SetName in pairs(db.profile) do
		ItemValue:RegisterSet(Group:new(SetName))
	end
end

function Group.prototype:init(Name, Sets)
    Group.super.prototype.init(self)
	
	if not db.profile[Name] then
		db.profile[Name] = {
			Sets = Sets or {},
		}
	end
	
	self.Name = Name
	self.Sets = db.profile[Name].Sets
	
	self.Options = {
		type = "group",
		name = Name,
		desc = string.format(L["Settings for Set \"%s\"."], Name),
		args = {
			sets = {
				order = 100,
				type  = "header",
				name  = L["Set Values"],
			},
		},
	}
	
	ItemValue:RegisterEvent("ItemValue_SetsChanged", function() self:Update() end)
end

function Group.prototype:Update()
	local n = 100
	
	if not self.GetFunc then
		self.GetFunc = function(k)
			return self.Sets[k] or 0
		end
		self.SetFunc = function(k, v)
			v = tonumber(v)
			if v == 0 then v = nil end
			self.Sets[k] = v
			ItemValue:ClearCache() 
		end
	end
	
	-- Remove old sets
	for SetName, Options in pairs(self.Options.args) do
		if Options.order > 100 and not ItemValue.SetByName[SetName] then
			self.Options.args[SetName] = nil
			self.Sets[SetName] = nil
		end
	end
	
	-- Add new sets
	for _, Set in pairs(ItemValue.SetByName) do
		if Set ~= self and not self.Options.args[Set.Name] then
			n = n + 1
			self.Options.args[Set.Name] = {
				order	= n,
				type	= "text",
				name	= Set.Name,
				desc	= L["Defines the value for this set."],
				get		= self.GetFunc,
				set		= self.SetFunc,
				validate = tonumber,
				usage	= "<Number>",
				passValue = Set.Name,
			}
		end
	end
end

function Group.prototype:tostring()
	return "ItemValue Group " .. self.Name
end

function Group.prototype:OnDelete()
	db.profile[self.Name] = nil
end

function Group.prototype:GetEquipValue(eq)
	local Points = 0
	
	if eq then
		for SetName, SetValue in pairs(self.Sets) do
			if ItemValue.SetByName[SetName] then
				Points = Points + (ItemValue.SetByName[SetName]:GetEquipValue(eq) * SetValue)
			end
		end
	end
	
	return Points
end

function Group.prototype:GetItemValue(ItemString)
	return self:GetEquipValue(ItemBonusLib:ScanItem(ItemString, true))
end

function Group.prototype:Serialize()
	return {Type = "Group", Name = self.Name, Sets = self.Sets}
end

function Group:Deserialize(t)
	return Group:new(t.Name, t.Sets)
end

ItemValue:RegisterSetType("Group", Group)
